@@ -22,6 +22,12 @@ class Resource:
 
         if self.name == 'realm':
             return kc.admin()
+        elif self.name == 'clients/TODO-client_id/roles':
+            # TODO kcapi should be able to .build() same api as returned by client.roles()...
+            # This looks like URL routing logic is needed in kc.build()
+            # The "if self.name == 'realm'" - this is URL routing logic.
+            # We continue with this logic.
+            return params["client_roles_api"]
         else:
             return kc.build(realm=realm, resource_name=self.name)
 
@@ -34,6 +40,9 @@ class Resource:
     def publish(self, body):
         return ResourcePublisher(self.key, body).publish(self._resource_api)
 
+    def publish_object(self, body, single_resource):
+        return ResourcePublisher(self.key, body, single_resource).publish(self._resource_api)
+
     def remove(self, body):
         id = self.get_resource_id(body)
         if id:
